
# Copyright 1996 Veritas Software

# Outline list widgets
# This source works, but probably needs some refinement, such as
# enabling font choices, handling the images (xbm files), documentation,
# performance tweeks, etc.

import rivet, types
import string
from rivetsel import selection_list

offsetwidth=20
font1 = ('-font', '-*-avantgarde-book-r-*-*-12-*-*-*-*-*-*-*')
font2 = ('-font', '-*-avantgarde-demi-r-*-*-12-*-*-*-*-*-*-*')
size = 18 # size of a single list item including gaps

class olistitem:
	isa_dir = 0
	def __init__(self, container):
		self.container = container
		self.rootview = container.rootview
		self.__widget = rivet.canvas(container,
			yscrollincrement=0, height=16, width=1000,
			bg='white',
			highlightthickness=0)
		me = self.canvas = self.__widget
		rivet.pack(me, expand=1, fill='x')
		me.yview('moveto', 0)
		me.create('rectangle', 16, -500, 1000, 1000,
			  outline='', fill='white', tag='background')
		me.tag_bind('all','<1>', (self.rootview.clear_selection, self))
		me.tag_bind('all', '<2>', self.adjust_cb)

	def decorate(self):
		pass

	def select_cb(self):
		self.rootview.clear_selection(self)
		# for later...
		#self.rootview.set_header()

	def adjust_cb(self):
		if self.rootview.is_selected(self):
			self.rootview.del_selection(self)
		else:
			self.rootview.add_selection(self)
			# for later...
			#self.rootview.set_header()

	def illustrate_selected(self):
		self.canvas.itemconfigure('background', fill='lightblue')

	def illustrate_normal(self):
		self.canvas.itemconfigure('background', fill='white')

	def remove(self):
		if self.rootview.is_selected(self):
			self.rootview.del_selection(self)
		rivet.destroy(self)

class olistitemdir:
	isa_dir = 1
	_superinit = olistitem.__init__.im_func
	decorate = olistitem.decorate.im_func
	select_cb = olistitem.select_cb.im_func
	adjust_cb = olistitem.adjust_cb.im_func
	illustrate_selected = olistitem.illustrate_selected.im_func
	illustrate_normal = olistitem.illustrate_normal.im_func
	_superremove = olistitem.remove.im_func

	def __init__(self, container):
		self.childlist = []
		self.container = container
		self.rootview = container.rootview
		self.offset = container.offset+container.rootview.offset
		self.__widget = rivet.frame(container)
		temp = self.__widget
		self._superinit(self)
		self.__widget = temp
		rivet.pack(self.__widget, expand=1, fill='x')
		self.isopen = -1 # init state only
		self.illustrate_closed()

	def remove(self):
		self.unpopulate()
		self._superremove()

	def illustrate_open(self):
		self.isopen = 1
		self.canvas.delete('triangle')
		me = self.canvas
		me.create('bitmap', 1, 8,
			bitmap='@openingarrow.xbm',
			anchor='w', tag='triangle')
		rivet.update()
		self.populate()
		me.delete('triangle')
		me.create('bitmap', 1, 8,
			bitmap='@downarrow.xbm',
			anchor='w', tag='triangle')
		me.tag_bind('triangle', '<1>', self.illustrate_closed)

	def illustrate_closed(self):
		me = self.canvas
		me.delete('triangle')
		me.create('bitmap', 1, 8,
			bitmap='@leftarrow.xbm',
			anchor='w', tag='triangle')
		me.tag_bind('triangle', '<1>', self.illustrate_open)
		self.isopen = 0
		self.unpopulate()

	def populate(self):
		# override this function to expand list
		pass

	def unpopulate(self):
		if len(self.childlist) == 0:
			return
		rivet.pack.forget(self.childlist)
		for child in self.childlist:
			child.remove()
		self.childlist = []

class olist(selection_list):
	def __init__(self, parent):
		selection_list.__init__(self)
		self.childlist = []
		self.rootview = self
		self.offset = offsetwidth
		self.__widget = rivet.frame(parent)
		self.clipframe = rivet.frame(self,
				bg='white', height=200, width=500)
		self.sbar = rivet.scrollbar(self,
			'command',self.outline_scroll)
		self.outline = rivet.frame(self.clipframe)
		self.outline.rootview = self
		self.outline.offset = offsetwidth

		rivet.pack(self.sbar, side='right', fill='y')
		rivet.pack(self.clipframe, expand=1, fill='both')
		rivet.place(self.outline, x=0, y=0)

		self.clipframe.bind('<Configure>', self.resize, 1)
		self.outline.bind('<Configure>', self.scrollbar_set, 1)

	def resize(self):
		global size
		# It is important to remember that winy is a negative offset
		cliph = float(rivet.winfo.height(self.clipframe))
		winh = float(rivet.winfo.height(self.outline))
		winy = float(rivet.winfo.y(self.outline))

		if cliph > winh:
			# outline fits in the viewable space, set it at top
			self.outline.place('-y', 0)
		elif winh + winy < cliph:
			# adjust so there is no gap at bottom (newy is negative)
			newy = -winh+cliph
			# make sure that all of last item will be visible,
			# then round so that first item is completely visible
			newy = newy-(size/2)+1
			newy = round(newy/size)*size
			self.outline.place(y=newy)

		self.scrollbar_set()
		
	def scrollbar_set(self):
		cliph = float(rivet.winfo.height(self.clipframe))
		winh = float(rivet.winfo.height(self.outline))

		start = -float(rivet.winfo.y(self.outline))/winh
		self.sbar.set(start, start + cliph/winh)

	def outline_scroll(self, args):
		global size
		cliph = float(rivet.winfo.height(self.clipframe))
		winh = float(rivet.winfo.height(self.outline))

		if cliph > winh:
			newy = 0.0
		else:
			if args[0] == 'moveto':
				newy = args[1] * -winh
			else:
				current = float(rivet.winfo.y(self.outline))
				if args[2] == 'units':
					newy = current + args[1] * -size # gack
				else: # pages
					newy = current - cliph * args[1]

			# prevent scroll off bottom, gap at top
			if newy > 0: newy = 0

			# prevent scroll off top, gap at bottom
			if newy + winh < cliph:
				newy = -winh+cliph
				newy = newy-(size/2)+1

			newy = round(newy/size)*size

		self.outline.place('-y', newy)
		self.scrollbar_set()
		

	def notify_selection(self, changeobj, addordel):
		if addordel:
			changeobj.illustrate_selected()
		else:
			changeobj.illustrate_normal()
		# call super...
		selection_list.notify_selection(self, changeobj, addordel)

	def populate(self):
		pass

	def unpopulate(self):
		pass

####

class stringitem(olistitem):
	def __init__(self, container, text):
		self.isa_dir = 0
		self.text = text
		olistitem.__init__(self, container)
		self.decorate()

	def decorate(self):
		if self.isa_dir:
			offset = self.container.offset-self.rootview.offset
			font = font2
		else:
			offset = self.container.offset
			font = font1

		hoff = self.canvas.cget('height')/2
		self.canvas.create('text', offset, hoff,
				'-text', self.text,
                                 font, anchor='w', tag='name')

def stringpopulate(self, container):
	for item in self.subitems:
		if type(item) == types.StringType:
			# a simple item
			self.childlist.append(
				stringitem( container, item ) )
		elif type(item) == types.ListType:
			# a list of items, first item is the name of this list
			self.childlist.append(
				stringitemdir( container, item[0], item[1:] ) )
		elif type(item) == types.TupleType:
			# an item, with a callback
			new = stringitem( container, item[0] )
			self.childlist.append(new)
			rivet.bind(new, '<Double-Button-1>', item[1] )

class stringitemdir(olistitemdir):
	def __init__(self, container, text, subitems):
		self.isa_dir = 1
		self.text = text
		self.subitems = subitems
		olistitemdir.__init__(self, container)
		self.decorate()

	def populate(self):
		stringpopulate(self, self)

	decorate = stringitem.decorate.im_func

class stringlist(olist):
	def __init__(self, parent):
		olist.__init__(self, parent)

	def populate(self):
		stringpopulate(self, self.outline)
	

def demo():
	def callback():
		print 'The magic object was double clicked!'

	t = rivet.init()
	l = [	'List Item 1',
		'List Item 2',
		'List Item 3',
		['List Item 4 (A Directory)',
			'List Item 4.1',
			'List Item 4.2',
			['List Item 4.3 (A Directory)',
				'List Item 4.3.1',
				('List Item 4.3.2 (has a callback)', callback)]],
		'List Item 5']
	sl = stringlist(t)
	sl.subitems = l
	sl.populate()
	rivet.pack(sl, expand=1, fill='both')
	rivet.mainloop()

if __name__ == '__main__':
	demo()

