
# Copyright 1996 Veritas Software 

# A class that maintains a list of objects that are "selected".
# - multiple objects may be on selection list
# - objects may be added, deleted individually
# - all objects may be cleared with one call

from rivetnotify import notify

class selection_list:
	def __init__(self):
		self.things={}
		self.n_selected=0

	def notify_selection(self, changeobj, addordel):
		pass

	def clear_selection(self, remain):
		# remove all but 'remain' from selected list
		for temp in self.things.keys():
			if temp != remain:
				self.del_selection(temp)
		if remain != None:
			if not self.things.has_key(remain):
				self.add_selection(remain)
			else:
				self.things[remain] = 1

	def add_selection(self, newobj):
		# perhaps add ability to add a list of objects
		if not self.things.has_key(newobj):
			self.n_selected = self.n_selected + 1
			self.things[newobj] = self.n_selected
			self.notify_selection(newobj, 1)
			return 1
		return 0

	def del_selection(self, delobj):
		if self.things.has_key(delobj):
			self.n_selected = self.n_selected - 1
			del self.things[delobj]
			self.notify_selection(delobj, 0)
			return 1
		return 0

	def is_selected(self, checkobj):
		if self.things.has_key(checkobj):
			return 1
		return 0

	def get_selected_list(self):
		return self.things.keys()

# This class permits multiple objects to get notification of a
# selection change.
class notify_selection_list(notify, selection_list):
	def notify_selection(self, changeobj, addordel):
		self.callclients(self)
	# maybe there should be a callclients with additional args


def test():
	# some test routines...
	class z1:
		pass
	class z2:
		pass

	print 'Running test...'
	a = z1()
	b = z2()

	s = selection_list()
	s.clear_selection(None)
	if s.get_selected_list() != []:
		print 'This should not happen(A)'

	s.add_selection(a)
	s.add_selection(b)
	if s.get_selected_list() != [a,b]:
		print 'This should not happen(B)'
	s.del_selection(a)
	s.del_selection(b)
	if s.get_selected_list() != []:
		print 'This should not happen(C)'

	s.add_selection(a)
	if not s.is_selected(a):
		print 'This should not happen(1)'
	if s.is_selected(b):
		print 'This should not happen(2)'
	if not s.is_selected(a):
		print 'This should not happen(2a)'
	if s.del_selection(b):
		print 'This should not happen(3)'
	if s.add_selection(a):
		print 'This should not happen(4)'

	s.clear_selection(a)
	if not s.is_selected(a):	
		print 'This should not happen(5)'

	s.clear_selection(b)
	if s.is_selected(a):	
		print 'This should not happen(6)'

	s.clear_selection(None)

	s.add_selection(a)
	s.clear_selection(None)

	s.clear_selection(a)
	if not s.is_selected(a):	
		print 'This should not happen(7)'


	s.clear_selection(None)

	print 'Test complete.'

if __name__ == '__main__':
	test()

